use std::{collections::HashMap, sync::Arc, time::Duration};
use tokio::sync::{watch, Mutex};
use tracing::info;

#[derive(Clone)]
pub struct Download {
    pub id: Option<String>,
    pub link: String,
}

pub struct InMemoryDownloadService {
    downloads:
        Mutex<HashMap<String, (Arc<Mutex<Download>>, tokio::sync::watch::Receiver<Download>)>>,
}

impl InMemoryDownloadService {
    pub fn new() -> Self {
        Self {
            downloads: Mutex::new(HashMap::new()),
        }
    }

    pub async fn add_download(&self, download: Download) -> anyhow::Result<Download> {
        let mut downloads = self.downloads.lock().await;

        let (tx, rx) = watch::channel(download.clone());

        downloads.insert(
            "key".to_string(),
            (Arc::new(Mutex::new(download.clone())), rx.clone()),
        );

        tokio::spawn({
            let d = download.clone();
            async move {
                loop {
                    info!("Sending event: {}", d.clone().id.unwrap());
                    let _ = tx.send(d.clone());
                    tokio::time::sleep(Duration::from_millis(300)).await;
                }
            }
        });

        Ok(download)
    }

    pub async fn get_download(&self, id: String) -> anyhow::Result<Option<Download>> {
        let downloads = self.downloads.lock().await;

        if let Some(d) = downloads.get(&id) {
            let download = d.0.lock().await;

            Ok(Some(download.clone()))
        } else {
            Ok(None)
        }
    }

    pub async fn subscribe_download(&self, id: String) -> tokio::sync::watch::Receiver<Download> {
        let downloads = self.downloads.lock().await;

        let download = downloads.get(&id).unwrap();

        download.1.clone()
    }
}
